@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.utils.encoding import force_text
+from django.forms.widgets import flatatt
 
-"""
-Handle HTML manipulation
-"""
+from .text import force_text, text_value
+
+
+# Handle HTML and CSS manipulation
 
 
 def split_css_classes(css_classes):
@@ -32,6 +33,26 @@ def remove_css_class(css_classes, css_class):
     Remove a CSS class from a string of CSS classes
     """
     remove = set(split_css_classes(css_class))
-    classes_list = [c for c in split_css_classes(css_classes)
-                    if c not in remove]
+    classes_list = [c for c in split_css_classes(css_classes) if c not in remove]
     return ' '.join(classes_list)
+
+
+def render_link_tag(url, rel='stylesheet', media='all'):
+    """
+    Build a link tag
+    """
+    return render_tag('link', attrs = {'href': url, 'rel': rel, 'media': media}, close=False)
+
+
+def render_tag(tag, attrs=None, content=None, close=True):
+    """
+    Render a HTML tag
+    """
+    builder = '<{tag}{attrs}>{content}'
+    if content or close:
+        builder += '</{tag}>'
+    return builder.format(
+        tag=tag,
+        attrs=flatatt(attrs) if attrs else '',
+        content=text_value(content),
+    )
